
import { Req } from '../http/request';
import { HttpConfig } from '../http/server';
import { HttpWebDependencies } from './server';
import { csp_headers } from '../http/content-security-policy';
import { FastifyInstance, RouteShorthandOptions } from 'fastify';
import { load_template } from '../utilities/mustache';

export function register_asset_endpoints(http_server: FastifyInstance, conf: HttpConfig, { color_themes }: HttpWebDependencies) {
	const opts: RouteShorthandOptions = {
		schema: { },
	};

	endpoint('forms.css', 'text/css; charset=utf-8');
	endpoint('popup.css', 'text/css; charset=utf-8');
	endpoint('structure.css', 'text/css; charset=utf-8');
	endpoint('typography.css', 'text/css; charset=utf-8');
	endpoint('color-theme-controls.css', 'text/css; charset=utf-8');
	endpoint('color-theme-controls.js', 'text/javascript');
	
	endpoint('weather.gov/styles.css', 'text/css; charset=utf-8');
	endpoint('weatherapi.com/styles.css', 'text/css; charset=utf-8');

	http_server.get('/themes.css', opts, async (req: Req, res) => {
		res.status(200);
		res.header('content-type', 'text/css; charset=utf-8');
		res.header('cache-control', 'public, max-age=3600');
		csp_headers(res, conf.exposed_url);
		return color_themes.css;
	});

	function endpoint(path: string, media_type: string) {
		http_server.get('/' + path, opts, async (req: Req, res) => {
			res.status(200);
			res.header('content-type', media_type);
			res.header('cache-control', 'public, max-age=3600');
			csp_headers(res, conf.exposed_url);
			return load_template(path);
		});
	}
}
