
import { pino } from 'pino';
import { create_weather_gov_provider, validate_weather_gov_conf, WeatherGovConfig } from './weather.gov';
import type { SecretValue } from '../conf';
import type { OutboundHttpProvider } from '../http/outbound';
import { create_weatherapi_com_provider, validate_weatherapi_com_conf, WeatherAPIComConfig } from './weatherapi.com';

export interface ServicesConfig {
	'weather.gov'?: WeatherGovConfig;
	'weatherapi.com'?: WeatherAPIComConfig;
	'openweathermap.org'?: {
		enabled: boolean;
		api_key: SecretValue;
	};
}

export type ServicesProvider = ReturnType<typeof create_services_provider>;

export function validate_services_conf(conf: unknown) : asserts conf is ServicesConfig {
	validate_weather_gov_conf(conf['weather.gov']);
	validate_weatherapi_com_conf(conf['weatherapi.com']);
}

export function create_services_provider(conf: ServicesConfig, logger: pino.Logger, outbound_http: OutboundHttpProvider) {
	const weather_gov = create_weather_gov_provider(conf['weather.gov'], logger, outbound_http);
	const weatherapi_com = create_weatherapi_com_provider(conf['weatherapi.com'], logger, outbound_http);

	const ready = Promise.all([
		weather_gov.ready,
		weatherapi_com.ready,
	]);

	return {
		ready,
		weather_gov,
		weatherapi_com,
	};
}
