
import { ContextDefinition } from 'jsonld';
import { HttpURL, ISOTimestamp, Timezone, URN } from '../../utilities/types';

export interface WeatherGovMeasure<Unit extends `wmoUnit:${string}` = `wmoUnit:${string}`> {
	unitCode: Unit;
	value: number;
}

export interface WeatherGovPoints {
	'@context': ContextDefinition;
	geometry: string;
	cwa: string;
	forecastOffice: HttpURL;
	gridId: string;
	gridX: number;
	gridY: number;
	forecast: HttpURL;
	forecastHourly: HttpURL;
	forecastGridData: HttpURL;
	forecastStations: HttpURL;
	relativeLocation: {
		city: string;
		state: string;
		geometry: string;
		distance: WeatherGovMeasure<'wmoUnit:m'>;
		bearing: WeatherGovMeasure<'wmoUnit:degree_(angle)'>;
	};
	forecastZone: HttpURL;
	county: HttpURL;
	fireWeatherZone: HttpURL;
	timeZone: Timezone;
	radarStation: string;
}

export interface WeatherGovForecast {
	'@context': ContextDefinition;
	geometry: string;
	updated: ISOTimestamp;
	units: 'us' | 'si';
	forecastGenerator: 'BaselineForecastGenerator' | 'HourlyForecastGenerator';
	generatedAt: ISOTimestamp;
	updateTime: ISOTimestamp;
	validTimes: string;
	elevation: WeatherGovMeasure<'wmoUnit:m'>;
	periods: WeatherGovForecastPeriod[];
}

export interface WeatherGovForecastPeriod {
	number: number;
	name: string;
	startTime: ISOTimestamp;
	endTime: ISOTimestamp;
	isDaytime: boolean;
	temperature: number;
	temperatureUnit: 'F' | 'C';
	temperatureTrend: unknown; // todo: what is this?
	probabilityOfPrecipitation: WeatherGovMeasure<'wmoUnit:percent'>;
	dewPoint: WeatherGovMeasure<'wmoUnit:degC'>;
	relativeHumidity: WeatherGovMeasure<'wmoUnit:percent'>;
	windSpeed: `${number} mph` | `${number} km/h`;
	windDirection: 'N' | 'NNW' | 'NW' | 'WNW' | 'W' | 'NNE' | 'NE' | 'ENE' | 'E'
		| 'S' | 'SSW' | 'SW' | 'WSW' | 'W' | 'SSE' | 'SE' | 'ESE' | 'E';
	icon: HttpURL;
	shortForecast: string;
	detailedForecast: string;
}

export interface WeatherGovAlerts {
	'@context': ContextDefinition;
	'@graph': {
		'@id': HttpURL;
		'@type': 'wx:Alert',
		id: URN<`oid:${string}`>;
		areaDesc: string;
		geometry: string;
		geocode: {
			SAME: `${number}`[];
			UGC: string[];
		};
		affectedZones: HttpURL[];
		references: unknown[]; // todo: what is this?
		sent: ISOTimestamp;
		effective: ISOTimestamp;
		onset: ISOTimestamp;
		expires: ISOTimestamp;
		ends: ISOTimestamp;
		status: 'Actual' | 'Exercise' | 'System' | 'Test' | 'Draft';
		messageType: 'Alert' | 'Update' | 'Cancel';
		category: string;
		severity: 'Extreme' | 'Severe' | 'Moderate' | 'Minor' | 'Unknown';
		certainty: 'Observed' | 'Likely' | 'Possible' | 'Unlikely' | 'Unknown';
		urgency: 'Immediate' | 'Expected' | 'Future' | 'Past' | 'Unknown';
		event: string;
		sender: string;
		senderName: string;
		headline: string;
		description: string;
		instruction: string;
		response: string;
		parameters: Record<string, string[]>;
	}[];
}
