
import { HttpURLSchemeless, ISODate, LocalDateTime, Timezone } from '../../utilities/types';

export interface WeatherAPIComLocation {
	name: string;
	region: string;
	country: string
	lat: number;
	lon: number;
	tz_id: Timezone;
	localtime_epoch: number;
	localtime: LocalDateTime;
}

export interface WeatherAPIComCurrentWeather {
	location: WeatherAPIComLocation;
	current: WeatherAPIComWeather & {
		last_updated_epoch: number;
		last_updated: LocalDateTime;
	};
}

export interface WeatherAPIComForecast extends WeatherAPIComCurrentWeather {
	forecast: {
		forecastday: {
			date: ISODate;
			date_epoch: number;
			day: {
				maxtemp_c: number;
				maxtemp_f: number;
				mintemp_c: number;
				mintemp_f: number;
				avgtemp_c: number;
				avgtemp_f: number;
				maxwind_mph: number;
				maxwind_kph: number;
				totalprecip_mm: number;
				totalprecip_in: number;
				totalsnow_cm: number;
				avgvis_km: number;
				avgvis_miles: number;
				avghumidity: number;
				daily_will_it_rain: number;
				daily_chance_of_rain: number;
				daily_will_it_snow: number;
				daily_chance_of_snow: number;
				condition: WeatherAPIComCondition;
				uv: number;
			};
			astro: {
				sunrise: string;
				sunset: string;
				moonrise: string;
				moonset: string;
				moon_phase: string;
				moon_illumination: `${bigint}`;
				is_moon_up: 0 | 1;
				is_sun_up: 0 | 1;
			};
			hour: WeatherAPIComHourForecast[];
		}[];
	};
}

export interface WeatherAPIComCondition {
	text: string;
	icon: HttpURLSchemeless;
	code: number;
}

export interface WeatherAPIComWeather {
	temp_c: number;
	temp_f: number;
	is_day: number;
	condition: WeatherAPIComCondition;
	wind_mph: number;
	wind_kph: number;
	wind_degree: number;
	wind_dir: string;
	pressure_mb: number;
	pressure_in: number;
	precip_mm: number;
	precip_in: number;
	humidity: number;
	cloud: number;
	feelslike_c: number;
	feelslike_f: number;
	vis_km: number;
	vis_miles: number;
	gust_mph: number;
	gust_kph: number;
	uv: number;
}

export interface WeatherAPIComHourForecast extends WeatherAPIComWeather {
	time_epoch: number;
	time: LocalDateTime;
	windchill_c: number;
	windchill_f: number;
	heatindex_c: number;
	heatindex_f: number;
	dewpoint_c: number;
	dewpoint_f: number;
	will_it_rain: number;
	chance_of_rain: number;
	will_it_snow: number;
	chance_of_snow: number;
}
