
import { ColorTheme, load_theme } from '@doc-utils/color-themes';
import { render_template } from './mustache';

export interface ColorThemeConfig {
	default: LightAndDarkTheme;
	less_contrast?: LightAndDarkTheme;
	more_contrast?: LightAndDarkTheme;
}

export interface LightAndDarkTheme {
	light: string;
	dark: string;
}

export type ColorThemeProvider = ReturnType<typeof create_color_theme_provider>;

export function create_color_theme_provider(conf: ColorThemeConfig) {
	let css: string;
	const ready = setup();

	return {
		get ready() {
			return ready;
		},
		get themes() {
			return structuredClone(conf);
		},
		get css() {
			return css;
		},
	};

	async function setup() {
		const view = {
			more_contrast: conf.more_contrast != null,
			less_contrast: conf.less_contrast != null,
		};

		const partials = {
			default_light: render_theme_css_properties(await load(conf.default.light)),
			default_dark: render_theme_css_properties(await load(conf.default.dark)),
			more_contrast_light: render_theme_css_properties(await load(conf.more_contrast?.light)),
			more_contrast_dark: render_theme_css_properties(await load(conf.more_contrast?.dark)),
			less_contrast_light: render_theme_css_properties(await load(conf.less_contrast?.light)),
			less_contrast_dark: render_theme_css_properties(await load(conf.less_contrast?.dark)),
		};

		css = await render_template('themes.css.mustache', view, partials);
	}
}

function load(theme: string) {
	if (theme) {
		return load_theme(theme);
	}

	return null;
}

function render_theme_css_properties(theme: ColorTheme) {
	return Object
		.entries(theme.colors)
		.map(([ name, color ]) => {
			return `--theme-${name.replace(/_/g, '-')}: ${color};`;
		})
		.join('\n') + '\n';
}
