
import { render } from 'mustache';
import { promises as fs } from 'fs';
import { resolve as path_resolve } from 'path';

const templates: Record<string, Promise<string>> = Object.create(null);
const template_path = path_resolve(__dirname, '../../templates');

export async function render_template(file: string, view: any, partials: Record<string, string> = Object.create(null), partial_files: Record<string, string> = Object.create(null)) {
	const template = await load_template(file);
	const partial_promises: Promise<void>[] = [ ];
	
	for (const [key, file] of Object.entries(partial_files)) {
		const promise = load_template(file).then((template) => {
			partials[key] = template;
		});

		partial_promises.push(promise);
	}

	await Promise.all(partial_promises);
	return render(template, view, partials);
}

export function load_template(file: string) {
	if (! templates[file]) {
		const path = path_resolve(template_path, file);
		templates[file] = fs.readFile(path, 'utf8');
	}

	return templates[file];
}
